/**
 * Copyright (c) 2017-2018 INCHRON GmbH
 * 
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     INCHRON GmbH - Meta Model design and implementation
 */
package com.inchron.realtime.root.model.autosar.util;

import com.inchron.realtime.root.Referrable;

import com.inchron.realtime.root.model.CallSequenceItem;
import com.inchron.realtime.root.model.ClockUser;
import com.inchron.realtime.root.model.Component;
import com.inchron.realtime.root.model.Event;
import com.inchron.realtime.root.model.Function;
import com.inchron.realtime.root.model.ModelObject;
import com.inchron.realtime.root.model.Schedulable;
import com.inchron.realtime.root.model.Spinlock;
import com.inchron.realtime.root.model.TraceObject;

import com.inchron.realtime.root.model.autosar.ARSystem;
import com.inchron.realtime.root.model.autosar.AutosarPackage;
import com.inchron.realtime.root.model.autosar.ClearEvent;
import com.inchron.realtime.root.model.autosar.Operation;
import com.inchron.realtime.root.model.autosar.OsAlarm;
import com.inchron.realtime.root.model.autosar.OsAlarmAction;
import com.inchron.realtime.root.model.autosar.OsApplication;
import com.inchron.realtime.root.model.autosar.OsConfig;
import com.inchron.realtime.root.model.autosar.OsCounter;
import com.inchron.realtime.root.model.autosar.OsEvent;
import com.inchron.realtime.root.model.autosar.OsExpiryPoint;
import com.inchron.realtime.root.model.autosar.OsIsr;
import com.inchron.realtime.root.model.autosar.OsResource;
import com.inchron.realtime.root.model.autosar.OsScheduleTable;
import com.inchron.realtime.root.model.autosar.OsSpinlock;
import com.inchron.realtime.root.model.autosar.OsTask;
import com.inchron.realtime.root.model.autosar.ResumeOSInterrupts;
import com.inchron.realtime.root.model.autosar.ServerCall;
import com.inchron.realtime.root.model.autosar.SuspendOSInterrupts;
import com.inchron.realtime.root.model.autosar.SwComponent;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notifier;

import org.eclipse.emf.common.notify.impl.AdapterFactoryImpl;

import org.eclipse.emf.ecore.EObject;

/**
 * <!-- begin-user-doc -->
 * The <b>Adapter Factory</b> for the model.
 * It provides an adapter <code>createXXX</code> method for each class of the model.
 * <!-- end-user-doc -->
 * @see com.inchron.realtime.root.model.autosar.AutosarPackage
 * @generated
 */
public class AutosarAdapterFactory extends AdapterFactoryImpl {
	/**
	 * The cached model package.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected static AutosarPackage modelPackage;

	/**
	 * Creates an instance of the adapter factory.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public AutosarAdapterFactory() {
		if (modelPackage == null) {
			modelPackage = AutosarPackage.eINSTANCE;
		}
	}

	/**
	 * Returns whether this factory is applicable for the type of the object.
	 * <!-- begin-user-doc -->
	 * This implementation returns <code>true</code> if the object is either the model's package or is an instance object of the model.
	 * <!-- end-user-doc -->
	 * @return whether this factory is applicable for the type of the object.
	 * @generated
	 */
	@Override
	public boolean isFactoryForType(Object object) {
		if (object == modelPackage) {
			return true;
		}
		if (object instanceof EObject) {
			return ((EObject)object).eClass().getEPackage() == modelPackage;
		}
		return false;
	}

	/**
	 * The switch that delegates to the <code>createXXX</code> methods.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected AutosarSwitch<Adapter> modelSwitch =
		new AutosarSwitch<Adapter>() {
			@Override
			public Adapter caseARSystem(ARSystem object) {
				return createARSystemAdapter();
			}
			@Override
			public Adapter caseClearEvent(ClearEvent object) {
				return createClearEventAdapter();
			}
			@Override
			public Adapter caseOperation(Operation object) {
				return createOperationAdapter();
			}
			@Override
			public Adapter caseOsAlarm(OsAlarm object) {
				return createOsAlarmAdapter();
			}
			@Override
			public Adapter caseOsAlarmAction(OsAlarmAction object) {
				return createOsAlarmActionAdapter();
			}
			@Override
			public Adapter caseOsApplication(OsApplication object) {
				return createOsApplicationAdapter();
			}
			@Override
			public Adapter caseOsConfig(OsConfig object) {
				return createOsConfigAdapter();
			}
			@Override
			public Adapter caseOsCounter(OsCounter object) {
				return createOsCounterAdapter();
			}
			@Override
			public Adapter caseOsEvent(OsEvent object) {
				return createOsEventAdapter();
			}
			@Override
			public Adapter caseOsExpiryPoint(OsExpiryPoint object) {
				return createOsExpiryPointAdapter();
			}
			@Override
			public Adapter caseOsIsr(OsIsr object) {
				return createOsIsrAdapter();
			}
			@Override
			public Adapter caseOsResource(OsResource object) {
				return createOsResourceAdapter();
			}
			@Override
			public Adapter caseOsScheduleTable(OsScheduleTable object) {
				return createOsScheduleTableAdapter();
			}
			@Override
			public Adapter caseOsSpinlock(OsSpinlock object) {
				return createOsSpinlockAdapter();
			}
			@Override
			public Adapter caseOsTask(OsTask object) {
				return createOsTaskAdapter();
			}
			@Override
			public Adapter caseResumeOSInterrupts(ResumeOSInterrupts object) {
				return createResumeOSInterruptsAdapter();
			}
			@Override
			public Adapter caseRunnable(com.inchron.realtime.root.model.autosar.Runnable object) {
				return createRunnableAdapter();
			}
			@Override
			public Adapter caseServerCall(ServerCall object) {
				return createServerCallAdapter();
			}
			@Override
			public Adapter caseSuspendOSInterrupts(SuspendOSInterrupts object) {
				return createSuspendOSInterruptsAdapter();
			}
			@Override
			public Adapter caseSwComponent(SwComponent object) {
				return createSwComponentAdapter();
			}
			@Override
			public Adapter caseReferrable(Referrable object) {
				return createReferrableAdapter();
			}
			@Override
			public Adapter caseModelObject(ModelObject object) {
				return createModelObjectAdapter();
			}
			@Override
			public Adapter caseSystem(com.inchron.realtime.root.model.System object) {
				return createSystemAdapter();
			}
			@Override
			public Adapter caseCallSequenceItem(CallSequenceItem object) {
				return createCallSequenceItemAdapter();
			}
			@Override
			public Adapter caseClockUser(ClockUser object) {
				return createClockUserAdapter();
			}
			@Override
			public Adapter caseEvent(Event object) {
				return createEventAdapter();
			}
			@Override
			public Adapter caseTraceObject(TraceObject object) {
				return createTraceObjectAdapter();
			}
			@Override
			public Adapter caseSchedulable(Schedulable object) {
				return createSchedulableAdapter();
			}
			@Override
			public Adapter caseProcess(com.inchron.realtime.root.model.Process object) {
				return createProcessAdapter();
			}
			@Override
			public Adapter caseSpinlock(Spinlock object) {
				return createSpinlockAdapter();
			}
			@Override
			public Adapter caseFunction(Function object) {
				return createFunctionAdapter();
			}
			@Override
			public Adapter caseComponent(Component object) {
				return createComponentAdapter();
			}
			@Override
			public Adapter defaultCase(EObject object) {
				return createEObjectAdapter();
			}
		};

	/**
	 * Creates an adapter for the <code>target</code>.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param target the object to adapt.
	 * @return the adapter for the <code>target</code>.
	 * @generated
	 */
	@Override
	public Adapter createAdapter(Notifier target) {
		return modelSwitch.doSwitch((EObject)target);
	}


	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.ARSystem <em>AR System</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.ARSystem
	 * @generated
	 */
	public Adapter createARSystemAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.ClearEvent <em>Clear Event</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.ClearEvent
	 * @generated
	 */
	public Adapter createClearEventAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.Operation <em>Operation</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.Operation
	 * @generated
	 */
	public Adapter createOperationAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.OsAlarm <em>Os Alarm</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.OsAlarm
	 * @generated
	 */
	public Adapter createOsAlarmAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.OsAlarmAction <em>Os Alarm Action</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.OsAlarmAction
	 * @generated
	 */
	public Adapter createOsAlarmActionAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.OsApplication <em>Os Application</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.OsApplication
	 * @generated
	 */
	public Adapter createOsApplicationAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.OsConfig <em>Os Config</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.OsConfig
	 * @generated
	 */
	public Adapter createOsConfigAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.OsCounter <em>Os Counter</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.OsCounter
	 * @generated
	 */
	public Adapter createOsCounterAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.OsEvent <em>Os Event</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.OsEvent
	 * @generated
	 */
	public Adapter createOsEventAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.OsExpiryPoint <em>Os Expiry Point</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.OsExpiryPoint
	 * @generated
	 */
	public Adapter createOsExpiryPointAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.OsIsr <em>Os Isr</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.OsIsr
	 * @generated
	 */
	public Adapter createOsIsrAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.OsResource <em>Os Resource</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.OsResource
	 * @generated
	 */
	public Adapter createOsResourceAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.OsScheduleTable <em>Os Schedule Table</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.OsScheduleTable
	 * @generated
	 */
	public Adapter createOsScheduleTableAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.OsSpinlock <em>Os Spinlock</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.OsSpinlock
	 * @generated
	 */
	public Adapter createOsSpinlockAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.OsTask <em>Os Task</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.OsTask
	 * @generated
	 */
	public Adapter createOsTaskAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.ResumeOSInterrupts <em>Resume OS Interrupts</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.ResumeOSInterrupts
	 * @generated
	 */
	public Adapter createResumeOSInterruptsAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.Runnable <em>Runnable</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.Runnable
	 * @generated
	 */
	public Adapter createRunnableAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.ServerCall <em>Server Call</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.ServerCall
	 * @generated
	 */
	public Adapter createServerCallAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.SuspendOSInterrupts <em>Suspend OS Interrupts</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.SuspendOSInterrupts
	 * @generated
	 */
	public Adapter createSuspendOSInterruptsAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.autosar.SwComponent <em>Sw Component</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.autosar.SwComponent
	 * @generated
	 */
	public Adapter createSwComponentAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.Referrable <em>Referrable</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.Referrable
	 * @generated
	 */
	public Adapter createReferrableAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.ModelObject <em>Object</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.ModelObject
	 * @generated
	 */
	public Adapter createModelObjectAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.System <em>System</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.System
	 * @generated
	 */
	public Adapter createSystemAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.CallSequenceItem <em>Call Sequence Item</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.CallSequenceItem
	 * @generated
	 */
	public Adapter createCallSequenceItemAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.ClockUser <em>Clock User</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.ClockUser
	 * @generated
	 */
	public Adapter createClockUserAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.Event <em>Event</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.Event
	 * @generated
	 */
	public Adapter createEventAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.TraceObject <em>Trace Object</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.TraceObject
	 * @generated
	 */
	public Adapter createTraceObjectAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.Schedulable <em>Schedulable</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.Schedulable
	 * @generated
	 */
	public Adapter createSchedulableAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.Process <em>Process</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.Process
	 * @generated
	 */
	public Adapter createProcessAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.Spinlock <em>Spinlock</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.Spinlock
	 * @generated
	 */
	public Adapter createSpinlockAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.Function <em>Function</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.Function
	 * @generated
	 */
	public Adapter createFunctionAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.Component <em>Component</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.Component
	 * @generated
	 */
	public Adapter createComponentAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for the default case.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @generated
	 */
	public Adapter createEObjectAdapter() {
		return null;
	}

} //AutosarAdapterFactory
